/* -*-comment-start: "//";comment-end:""-*-
 * GNU Mes --- Maxwell Equations of Software
 * Copyright © 2018 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 *
 * This file is part of GNU Mes.
 *
 * GNU Mes is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or (at
 * your option) any later version.
 *
 * GNU Mes is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Mes.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <mes/lib.h>

struct info
{
  int flag;
};
struct foo
{
  int length;
  char *string;
  struct info info;
};
struct foo stack[] = { {3, "foo", {11}}, {4, "baar", {12}} };

int
main ()
{
  oputs (stack[0].string);
  oputs ("\n");
  oputs (stack[1].string);
  oputs ("\n");
  struct foo *top = &stack[1];
  int i;
  int j;
  i = (top--)->info.flag;
  top++;
  j = (--top)->info.flag;

  if (i - j - 1)
    return 1;

  i = (top++)->info.flag;
  top--;
  j = (++top)->info.flag;

  if (i - j != -1)
    return 2;

  return 0;
}
